/*
*   Tag-around-text-updater
*
*   The MIT License (MIT)

    Copyright (c) 2011-2018 Twitter, Inc.
    Copyright (c) 2011-2018 The Bootstrap Authors

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.

* */


/*
*   Private part : Error Handler
* */
function TagAroundTextUpdaterError() {
    let temp = Error.apply(this, arguments);
    temp.name = this.name = 'TagAroundTextUpdaterError';
    this.message = temp.message;
    if (Object.defineProperty) {
        // getter for more optimizy goodness
        /*this.stack = */
        Object.defineProperty(this, 'stack', {
            get: function () {
                return temp.stack
            },
            configurable: true // so you can change it if you want
        })
    } else {
        this.stack = temp.stack
    }
}

//inherit prototype using ECMAScript 5 (IE 9+)
TagAroundTextUpdaterError.prototype = Object.create(Error.prototype, {
    constructor: {
        value: TagAroundTextUpdaterError,
        writable: true,
        configurable: true
    }
});



/*
*     Private part : supporting functions
* */
const ValidObj = {

    isStrValid(sth) {
        if (typeof sth === 'string') {
            return true;
        }
        return false;
    },


    isBoolValid(sth) {
        if (typeof sth === 'boolean') {
            return true;
        }
        return false;
    },

};
const UtilObj = {

    replaceBetween(from, start, end, what) {
        return from.substring(0, start) + what + from.substring(end);
    },

    escapeRegex(s) {
        return s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
    }
};
const ProcessObj = {

    core(startTag, endTag, where, finalWhat, caseSensitive) {

        let tagAreaMatches = [];
        let whatMatches = [];

        if(caseSensitive && caseSensitive === true){
            caseSensitive = 'g';
        }else{
            caseSensitive = 'gi'; // default
        }

        /*
        *  Description : 'rx1' is a regex to distill all tag areas.
        * */

        let rx1 = new RegExp('<[^>]+>', 'g');

        let match = {};
        while ((match = rx1.exec(where)) !== null) {
            tagAreaMatches.push({
                'value': match[0],
                'index': match.index,
                'i2' : match.index + match[0].length
            })
        }

        /*
        *  Description : 'rx2' is a regex to distill all areas where 'what' exists.
        * */

        let rx2 = new RegExp(finalWhat, caseSensitive);

        let match2 = {};
        while ((match2 = rx2.exec(where)) !== null) {
            whatMatches.push({
                'value': match2[0],
                'index': match2.index,
                'i2' : match2.index + match2[0].length
            })
        }



        tagAreaMatches = tagAreaMatches.reverse();
        whatMatches = whatMatches.reverse();


        /*
        *   We don't need 'whats' inside tags.
        * */
        for(let a=0; a < whatMatches.length; a++){
            for(let b=0; b < tagAreaMatches.length; b++){
                if (whatMatches[a].index > tagAreaMatches[b].index &&
                    whatMatches[a].index + whatMatches[a].value.length < tagAreaMatches[b].index + tagAreaMatches[b].value.length) {

                    delete whatMatches[a];
                    break;
                }
            }
        }


        //console.log(tagAreaMatches);
        //console.log(whatMatches);

        /*
        *   Now, just change all target texts outside tags.
        * */
        whatMatches.forEach(function (val, idx) {
            where = UtilObj.replaceBetween(where, val.index, val.index + val.value.length, startTag + val.value + endTag);
        });

        return where;
    }

};

/*
*    Public part
* */
class TagAroundTextUpdater {

    /*
    *  The prefix here 'Store' means 'Create' if the key doesn't exist, or 'Update'.
    * */
    insertByWhat(startTag, endTag, what, where, caseSensitive) {

        try {

            if(!ValidObj.isStrValid(startTag)){
                throw new TagAroundTextUpdaterError("Only a string type is supported for startTag");
            }
            if(!ValidObj.isStrValid(endTag)){
                throw new TagAroundTextUpdaterError("Only a string type is supported for endTag");
            }
            if(!ValidObj.isStrValid(what)){
                throw new TagAroundTextUpdaterError("Only a string type is supported for 'what'");
            }
            if(!ValidObj.isStrValid(where)){
                throw new TagAroundTextUpdaterError("Only a string type is supported for 'where'");
            }
            if(!ValidObj.isStrValid(startTag)){
                throw new TagAroundTextUpdaterError("Only a string type is supported for startTag");
            }

            if((typeof(caseSensitive) !== "undefined" && caseSensitive !== null)  && !ValidObj.isBoolValid(caseSensitive)){
                throw new TagAroundTextUpdaterError("If 'caseSensitive' is indicated, it must be a boolean type.");
            }

            startTag = startTag.trim();
            endTag = endTag.trim();
            what = what.trim();
            where = where.trim();

            let finalWhat = UtilObj.escapeRegex(what);

            where = ProcessObj.core(startTag, endTag, where, finalWhat, caseSensitive);

            return where;

        } catch (e) {

            console.log(e);

        } finally {

            return where;

        }

    }

    insertByType(startTag, endTag, type, where) {

        try {

            if(!ValidObj.isStrValid(startTag) || !ValidObj.isStrValid(endTag) || !ValidObj.isStrValid(type) || !ValidObj.isStrValid(where)){
                throw new TagAroundTextUpdaterError("Only a string type is supported for parameters");
            }

            startTag = startTag.trim();
            endTag = endTag.trim();
            type = type.trim();
            where = where.trim();

            switch (type){
                case 'integer':
                    type = '[0-9]+';
                    break;

                case 'email':
                    type =  '(([^<>()\\[\\]\\.,;:\\s@\\"]+(\\.[^<>()\\[\\]\\.,;:\\s@\\"]+)*)|(\\".+\\"))@(([^<>()[\\]\\.,;:\\s@\\"]+\\.)+[^<>()[\\]\\.,;:\\s@\\"]{2,})';
                    break;

                default :
                    throw new TagAroundTextUpdaterError("An unknown type has been detected.");
            }

            where = ProcessObj.core(startTag, endTag, where, type);

            return where;

        } catch (e) {

            console.log(e);

        } finally {

            return where;

        }

    }

}

export default TagAroundTextUpdater;

